﻿//import { marker } from "leaflet";

// map heler for leaflet

namespace maphelper {
    export class MapType {
        constructor(public shortName: string, public name: string, public url: string, public minZoom: number, public maxZoom: number
        ) {
        }
    }
    export class MyMarkerPoint {
        constructor(public latitude: number, public longitude:number, public explanation: string, public url: string
        ) {
        }
    }

    export var maps: MapType[] = [
        new MapType("r", "地理院地図","https://cyberjapandata.gsi.go.jp/xyz/std/{z}/{x}/{y}.png",2,18),
        new MapType("a", "空中写真", "https://cyberjapandata.gsi.go.jp/xyz/seamlessphoto/{z}/{x}/{y}.jpg", 2, 18),
        new MapType("b", "2007年～", "https://cyberjapandata.gsi.go.jp/xyz/ort/{z}/{x}/{y}.jpg", 14, 18),
        new MapType("c", "2004年～", "https://cyberjapandata.gsi.go.jp/xyz/airphoto/{z}/{x}/{y}.png", 15, 18),

        new MapType("d", "1988～1990年", "https://cyberjapandata.gsi.go.jp/xyz/gazo4/{z}/{x}/{y}.jpg", 10, 17),
        new MapType("e", "1984～1986年", "https://cyberjapandata.gsi.go.jp/xyz/gazo3/{z}/{x}/{y}.jpg", 10, 17),
        new MapType("f", "1979～1983年", "https://cyberjapandata.gsi.go.jp/xyz/gazo2/{z}/{x}/{y}.jpg", 10, 17),
        new MapType("g", "1974～1978年", "https://cyberjapandata.gsi.go.jp/xyz/gazo1/{z}/{x}/{y}.jpg", 10, 17),

        new MapType("i", "1961～1969年", "https://cyberjapandata.gsi.go.jp/xyz/ort_old10/{z}/{x}/{y}.png", 10, 17),
        new MapType("j", "1945～1950年", "https://cyberjapandata.gsi.go.jp/xyz/ort_USA10/{z}/{x}/{y}.png", 10, 17),
        new MapType("k", "1936年頃", "https://cyberjapandata.gsi.go.jp/xyz/ort_riku10/{z}/{x}/{y}.png", 13, 18),
        new MapType("l", "色別標高図", "https://cyberjapandata.gsi.go.jp/xyz/relief/{z}/{x}/{y}.png", 5, 15),
        new MapType("m", "陰影起伏図", "https://cyberjapandata.gsi.go.jp/xyz/hillshademap/{z}/{x}/{y}.png", 2, 16),
    ];
}

function getAllMaps() {
    return maphelper.maps;
}
function getTargetMaps(shortName: string) {
    if (shortName == "h") return maphelper.maps[1]; // force to return "a"
    for (var i = 0; i < maphelper.maps.length; i++) {
        if (maphelper.maps[i].shortName == shortName) return maphelper.maps[i];
    }
    return maphelper.maps[0];
}

function createGM(targetElementId: string, shortMapName: string, xSize: string, ySize: string, centerLatitude: number, centerLongitude: number, zoomLevel: number, markers: maphelper.MyMarkerPoint[], isLine: boolean, afterProc: (mymap: L.Map, firstMarker: L.Marker, baseLayers: any, shortNameToLayer:any) => void = null) {
    setTimeout(() => {
        let map = getTargetMaps(shortMapName);
        if (!xSize) xSize = "100%";
        $("#" + targetElementId).css("width", xSize);
        if (!ySize) ySize = "20em";
        $("#" + targetElementId).css("height", ySize);
        if (!zoomLevel) zoomLevel = 13;
        if (map.maxZoom < zoomLevel) zoomLevel = map.maxZoom;
        if (map.minZoom > zoomLevel) zoomLevel = map.minZoom;
        let mymap = L.map(targetElementId).setView([centerLatitude, centerLongitude], zoomLevel);
        //let options: any = new Object();
        //options.maxZoom = map.maxZoom;
        //options.minZoom = map.minZoom;

        let baseLayers: any = new Object();
        let shortNameToLayer: any = new Object();
        let maps = getAllMaps();
        for (var i = 0; i < maps.length; i++) {
            let options: any = new Object();
            options.maxZoom = maps[i].maxZoom;
            options.minZoom = maps[i].minZoom;
            //options.id = targetElementId + "_sel" + i.toString();
            baseLayers[maps[i].name] = L.tileLayer(maps[i].url, options);
            shortNameToLayer[maps[i].shortName] = baseLayers[maps[i].name];
            if (maps[i].shortName == shortMapName) {
                baseLayers[maps[i].name].addTo(mymap);
            }
        }
        L.control.layers(baseLayers).addTo(mymap);
        //mymap.attributionControl.addAttribution('This is attr <a href="http://www.piedey.co.jp/">LINK</a>');

        if (!markers || markers.length == 0) {
            markers = [
                new maphelper.MyMarkerPoint(centerLatitude, centerLongitude, null, null)
            ];
        }

        if (isLine && markers.length >= 2) {
            // line mode 
            let latlngs: L.LatLng[] = [];
            for (var i = 0; i < markers.length; i++) {
                latlngs.push(new L.LatLng(markers[i].latitude, markers[i].longitude));
            }
            L.polyline(latlngs, { color: 'red' }).addTo(mymap);
        }
        // marker/line mode common
        let lmarkers: L.Marker[]=[];
        for (var i = 0; i < markers.length; i++) {
            let options: any = new Object();
            if (markers[i].explanation) options.title = markers[i].explanation;
            let marker = L.marker([markers[i].latitude, markers[i].longitude], options).addTo(mymap);
            lmarkers.push(marker);
            let url = markers[i].url;
            let m: any = marker;
            let tooltip = String.fromCharCode('A'.charCodeAt(0) + i);
            if (markers[i].explanation && markers[i].explanation!="*") {
                tooltip += ": " + markers[i].explanation;
            }
            m.bindTooltip(tooltip, { permanent: true, interactive: !!url }); //.openTooltip();
            if (url) marker.on('click', () => {
                location.href = url;
            });
        }
        if( markers.length >= 2 ){
            mymap.fitBounds(L.featureGroup(lmarkers).getBounds());
        }
        if (afterProc) {
            // for AdminWrite mode
            afterProc(mymap, lmarkers[0], baseLayers, shortNameToLayer);
        } else {
            // for normal mode
            // create additional UI
            let p = document.createElement('p');
            $(p).append("[");
            let a1 = document.createElement('a');
            $(a1).attr("href", "#");
            $(a1).click(() => {
                mymap.setView([centerLatitude, centerLongitude], zoomLevel);
            });
            $(a1).text("RESET");
            $(p).append(a1);
            $(p).append("]");
            $(p).append("[");
            let a2 = document.createElement('a');
            $(a2).attr("href", "#");
            $(a2).click(() => {
                var group = L.featureGroup(lmarkers);
                mymap.fitBounds(group.getBounds());
            });
            $(a2).text("VIEW ALL");
            $(p).append(a2);
            $(p).append("]");

            for (var i = 0; i < markers.length; i++) {
                $(p).append("[");
                let a3 = document.createElement('a');
                $(a3).attr("href", "#");
                let m = lmarkers[i];
                $(a3).click(() => {
                    var group = L.featureGroup([m]);
                    mymap.fitBounds(group.getBounds());
                });
                let tooltip = String.fromCharCode('A'.charCodeAt(0) + i);
                if (markers[i].explanation && markers[i].explanation != "*") {
                    tooltip += ": " + markers[i].explanation;
                }
                $(a3).text(tooltip);
                $(p).append(a3);
                $(p).append("]");
            }
            $("#" + targetElementId).after(p);
        }
    }, 0);
}
function initForAdminWriteOnly() {
    $("#MiscPanelActivator").click(() => {
        $("#miscPanel").css("display", "block");
        $("#MiscPanelActivator").css("display", "none");
    });
    $("#GMPickerActivator").click(() => {
        $("#map_frame").css("display", "block");
        $("#GMPickerActivator").css("display", "none");
        let lat = 35.68093426903065;
        let lon = 139.76703643798828;
        let zoom = 13;
        let shortName = "r";
        if (localStorage.getItem("lat")) lat = Number(localStorage.getItem("lat"));
        if (localStorage.getItem("lon")) lon = Number(localStorage.getItem("lon"));
        if (localStorage.getItem("zoom")) zoom = Number(localStorage.getItem("zoom"));
        if (localStorage.getItem("shortName")) shortName = localStorage.getItem("shortName");
        createGM("map_canvas", shortName, "500px", "300px", lat, lon, zoom, null, false, (mymap, marker, baseLayers, shortNameToLayer) => {
            let updateMessages = () => {
                marker.setLatLng(mymap.getCenter());
                let myshortName = null;
                for (var key in baseLayers) {
                    if (mymap.hasLayer(baseLayers[key])) {
                        for (let shortNameEx in shortNameToLayer) {
                            if (shortNameToLayer[shortNameEx] == baseLayers[key]) {
                                myshortName = shortNameEx;
                                break;
                            }
                        }
                        if (myshortName) break;
                    }
                }
                var center = mymap.getCenter();
                $("#message1").text("$$gmp ,," + center.lat + "," + center.lng
                    + "," + mymap.getZoom() + "," + myshortName);
                $("#message2").text("," + center.lat + "," + center.lng + ",*,");
                lat = mymap.getCenter().lat;
                lon = mymap.getCenter().lng;
                zoom = mymap.getZoom();
                shortName = myshortName;
            };
            $(window).bind("beforeunload", function () {
                updateMessages();
                localStorage.setItem("lat", lat.toString());
                localStorage.setItem("lon", lon.toString());
                localStorage.setItem("zoom", zoom.toString());
                localStorage.setItem("shortName", shortName);
            });
            mymap.on('moveend', (e) => {
                updateMessages();
            });
            $("#insertGMP").click(() => {
                updateMessages();
                $("#TextBoxBody").val($("#TextBoxBody").val() + $("#message1").text());
            });
            $("#addPoint").click(() => {
                updateMessages();
                $("#TextBoxBody").val($("#TextBoxBody").val() + $("#message2").text());
            });
            $("#comeHere").click(() => {
                if (navigator.geolocation) {
                    navigator.geolocation.getCurrentPosition((position) => {
                        mymap.setView([position.coords.latitude, position.coords.longitude]);
                    }, (error) => {
                        var errorMessage = {
                            0: "原因不明のエラーが発生しました…。",
                            1: "位置情報の取得が許可されませんでした…。",
                            2: "電波状況などで位置情報が取得できませんでした…。",
                            3: "位置情報の取得に時間がかかり過ぎてタイムアウトしました…。",
                        };
                        // エラーコードに合わせたエラー内容をアラート表示
                        alert(errorMessage[error.code]);
                    });
                }
                else {
                    // Geolocation APIに対応していない
                    alert("あなたの端末では、現在位置を取得できません。");
                }
            });
        });
    });
}
var allInitializers: ((event) => void)[] = [];
function kickAllInitializers() {
    for (var i = 0; i < allInitializers.length; i++) {
        allInitializers[i](null);
    }
    allInitializers = [];
}
